"""Top-level package for pydeflate."""

__author__ = """Jorge Rivera"""
__email__ = 'jorge.rivera@one.org'
__version__ = '0.1.3'


from pydeflate.deflate.deflate import deflate_data

from pydeflate.utils import update_data
